""" Python Application Launcher
Copyright (C) 2002 James ''Wez''Weatherall

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""

import sys

def _stderr(text):
    sys.stderr.write("%s: %s\n" % (pylauname, text))

def _stdout(text):
    sys.stdout.write("%s: %s\n" % (pylauname, text))

# - If Pylau is run in its own right then it will run as
#   the server, rather than trying to use an existing one
if __name__ == "__main__":
    pylauname = "pylau-srv"
    
    import socket
    import os

    _stderr("starting server")

    # - Create the pylau socket dir if required
    sockdir = os.path.join("/tmp",".pylau")
    if not os.access(sockdir, 0):
        _stderr("creating pylau directory")
        os.mkdir(sockdir)
        os.chmod(sockdir, 0777)

    # - Establish the name of the Pylau socket
    try:
        pipename = os.path.join(sockdir, os.environ["DISPLAY"])
    except:
        _stderr("no X11 display")
        sys.exit()

    # - Destroy any existing pylau socket
    try: os.unlink(pipename)
    except: pass

    # - Open the socket to listen for requests on
    _stderr("creating socket")
    try:
        conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        conn.bind(pipename)
        conn.listen(5)
    except:
        _stderr("unable to bind socket %s" % sys.exc_info()[1])
        sys.exit(1)

    # - Import Gtk
    import gtk
    _stderr("imported Gtk+")

    _test_env = ["USERNAME", "ENV", "PYTHONPATH", "HOSTTYPE",
                 "LOGNAME", "USER", "INPUTRC", "HOME", "PATH",
                 "LD_LIBRARY_PATH", "DISPLAY", "HOSTNAME"]

    # - Internal functions to launch apps correctly(!)
    def _launch(conn, info):
        try:
            sock = conn.accept()[0]
            _stderr("accepted connection")
            file = sock.makefile()
            argv = eval(file.readline())
            environ = eval(file.readline())
            for key in _test_env:
                if ((environ.has_key(key) and os.environ.has_key(key)) and
                    (environ[key] != os.environ[key])):
                    sock.send(repr(0)+"\n")
                    return
            _stderr("launch %s" % str(argv))
        except:
            _stderr("error reading request")
            sock.send(repr(0)+"\n")
            return

        try:
            import imp
            path, name = os.path.split(argv[0])
            name, ext = os.path.splitext(name)
            if path: paths = [path]
            else: paths = string.split(os.environ["PATH"],":")
            fd, pathname, desc = imp.find_module(name, paths)
            app = imp.load_module(name, fd, pathname, desc)
            try:
                app.Main(argv)
            except:
                _stderr("script %s has no Main(argv)" % name)
                sock.send(repr(0)+"\n")
                return
        except:
            _stderr("unable to launch:%s:%s" % (sys.exc_info()[0], sys.exc_info()[1]))
            sock.send(repr(0)+"\n")
            return

        try: sock.send(repr(1)+"\n")
        except: pass

    # *** Dodgy GTK hacking
    def _events_pending():
        if _gtk_mainloop_count: return 0
        return _gtk_events_pending()
    def _mainiteration():
        if _gtk_mainloop_count: return 0
        return _gtk_mainiteration()

    # *** Safer GTK hacking
    def _mainloop():
        global _gtk_mainloop_count
        old = _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count + 1

    def _mainquit(*args):
        global _gtk_mainloop_count
        _gtk_mainloop_count = _gtk_mainloop_count - 1

    # - Periodic inactivity check, to allow pylau to quit if unused
    #   Especially important if two instances start simultaneously.
    #   One instance will usually die after about a minute.
    def _timeout_idle():
        global _gtk_mainloop_count
        if not _gtk_mainloop_count: _gtk_mainquit()
        return 1

    # - Rewire gtk... !!!
    _gtk_mainloop_count = 0
    _gtk_mainloop = gtk.mainloop
    _gtk_mainquit = gtk.mainquit

    _gtk_mainiteration=gtk.mainiteration
    _gtk_events_pending=gtk.events_pending
    gtk.mainiteration=_mainiteration
    gtk.events_pending=_events_pending

    gtk.mainloop = _mainloop
    gtk.mainquit = _mainquit

    # - Hook us in to Gtk's select() calls
    gtk.input_add(conn, gtk.GDK.INPUT_READ, _launch)

    # - Hook us into the system timer, so we can quit after a period of inactivity
    gtk.timeout_add(60000, _timeout_idle)

    # - Put pylau in the system module table
    sys.modules["pylau"] = sys.modules[__name__]
    sys.stdout.write("pylau-status:OK\n")
    sys.stdout.flush()
    _stderr("OK")

    # - Rewire stdout and stderr to /dev/null...
    #sys.stdout = open("/dev/null", "w")
    #sys.stderr = open("/dev/null", "w")

    # - Run as Pylau until there are no clients remaining
    _gtk_mainloop()

    sys.exit()

else:
    pylauname = "pylau-clt"

    # - Pylau imported as a module - attempt to re-launch this
    #   process inside an existing Pylau server

    if "--no-pylau" in sys.argv:
        _stderr("pylau disabled")
        sys.argv.remove("--no-pylau")
        raise ImportError

    # - We will only be imported once per interpreter
    #   This means we ALWAYS check for an existing instance
    #   of pylau and run one if none is found

    import socket
    import os

    # - Find the pylau socket dir
    sockdir = os.path.join("/tmp",".pylau")

    def _launch(command):
        # - Attempt to connect to an existing launcher
        conn = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        try:
            pipename = os.path.join(sockdir, os.environ["DISPLAY"])
        except:
            _stderr("no X11 display")
            raise ImportError

        try:
            # - Connect to pylau to launch the host script
            _stderr("locating pylau")
            conn.connect(pipename)
            _stderr("connected to server at %s" % pipename)

            try:
                command[0] = os.path.abspath(command[0])
                conn.send(repr(command)+"\n")
                conn.send(repr(os.environ)+"\n")
                f=conn.makefile()
                if eval(f.readline()):
                    _stderr("success")
                    return 1
            except socket.error:
                _stderr("failure")
                return 0
        except socket.error:
            _stderr("connect error: %s" % sys.exc_info()[1])
            return 0

    if _launch(sys.argv[:]):
        _stderr("launched in pylau")
        sys.exit()
    else:
        # - There is no existing pylau instance - start one
	import popen2

        # - Launch a new Pylau process
        _stderr("launching pylau")
        pylau = popen2.Popen3("python /usr/lib/python2.2/site-packages/pylau.py")
        while 1:
            line = pylau.fromchild.readline()
            if not line: raise ImportError
            if line == "pylau-status:OK\n":
                _stderr("server OK")
                if _launch(sys.argv[:]):
                    _stderr("quitting")
                    sys.exit()
                raise ImportError
        
                
